<?php

namespace InternetGalerie\Igshop2\Domain\Repository;

use TYPO3\CMS\Extbase\Persistence\Repository;
use TYPO3\CMS\Extbase\Persistence\QueryInterface;
use InternetGalerie\Igshop2\Domain\Model\Cart;
use InternetGalerie\Igshop2\Domain\Model\Currency;
use DateTime;
use TYPO3\CMS\Extbase\Persistence\ObjectStorage;
use TYPO3\CMS\Core\Context\Context;
use TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * The repository for Carts
 */
class CartRepository extends Repository
{
    protected $defaultCartings = [
        'uid' => QueryInterface::ORDER_DESCENDING,
    ];

    /*
     * public function saveToSession(\InternetGalerie\Igshop2\Domain\Model\Cart $cart = null) { $this->session->registerObject($cart, "cart"); } public function getFromSession() { $result = $this->session->getObjectByIdentifier("cart"); if (!$result) { $result = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance("\\InternetGalerie\\Igshop2\\Domain\\Model\\Cart"); } return $result; }
     */
    /**
     * gibt den aktuellen benutzerwarenkorb zurück.
     * existiert keins, dann wird ein neuer wk angelegt.
     * wurde ein wk abgeschickt, so wird ebenfalls ein neuer wk angelegt
     * @return Cart
     */
    public function getCurrentCart(Currency $currency = null)
    {
        $context = GeneralUtility::makeInstance(Context::class);
        //expects: if there is an active cart then session "shopCart" must have valid cart-uid
        //enshures: returns active cart, if there is no active cart, then one is created, cart-uid is stored to session "shopCart"

        $request = $GLOBALS['TYPO3_REQUEST'];
        $frontendUser = $request->getAttribute('frontend.user');
        $sessionCartId = $frontendUser ? intval($frontendUser->getKey('ses', 'shopCart')) : 0;
        $cart = null;
        if ($sessionCartId > 0) { //if cart is in session
            $query = $this->createQuery();
            $query->getQuerySettings()
            ->setRespectSysLanguage(false);
            //$query->getQuerySettings()->setLanguageOverlayMode(false);
            $query->matching($query->logicalAnd(
                $query->equals('submitted', 0),
                $query->equals('uid', $sessionCartId)
                //,                $query->equals('userid', 0)
            ));

            $queryResult = $query->execute();

            if ($queryResult->count() > 0) { //if cart valid, eg after submit is sassion!=0 but active cart not present.
                $cart = $queryResult->getFirst();
            }
        }

        if ($cart === null) {
            $cart = GeneralUtility::makeInstance(Cart::class);
            //$cart = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance("InternetGalerie\\Igshop2\\Domain\\Model\\Cart");
            if (is_object($currency)) {
                $cart->setCurrency($currency);
            }

            $cart->setOrderdate(new DateTime());

            if ($context->getPropertyFromAspect('frontend.user', 'isLoggedIn')) {
                $cart->setUserId($context->getPropertyFromAspect('frontend.user', 'id'));
            }
        }

        $this->persistCart($cart);

        /*
        // gute Idee -> aber dann crdate oben anpassen
        $deleteQuery = $this->createQuery();
        //$date = new \DateTime('midnight');
        $deleteQuery->matching(
        $deleteQuery->logicalAnd(
        $deleteQuery->lessThanOrEqual('crdate', time()-84600),
        $deleteQuery->equals("submitted", 0),
        $deleteQuery->equals("products", 0)
        )
        );
        $cartsToDelete = $deleteQuery->execute()->toArray();
        foreach($cartsToDelete AS $delete){
        $this->remove($delete);
        }*/

        return $cart;
    }

    /**
     * @return Cart
     */
    public function getCurrentCartForUser($feUser, $currency)
    {
        $request = $GLOBALS['TYPO3_REQUEST'];
        $frontendUser = $request->getAttribute('frontend.user');
        $sessionCartId = $frontendUser ? intval($frontendUser->getKey('ses', 'shopCart')) : 0;
        if ($sessionCartId > 0) { //if cart is in session and we have no current cart on the user
            $query = $this->createQuery();
            $query->getQuerySettings()
            ->setRespectSysLanguage(false);
            //$query->getQuerySettings()->setLanguageOverlayMode(false);
            $query->matching($query->logicalAnd($query->equals('submitted', 0), $query->equals('uid', $sessionCartId)));
            $queryResult = $query->execute();
            if ($queryResult->count() > 0) { //if cart valid, eg after submit is sassion!=0 but active cart not present.
                $cart = $queryResult->getFirst();
            }
        }

        if ($cart == null) {
            $cart = $feUser->getCurrentCart();
        }

        // Create new cart if there is none or if the current cart already got submitted
        if ($cart == null || (is_object($cart) && $cart->getSubmitted())) {
            $cart = GeneralUtility::makeInstance(Cart::class);
            if (is_object($currency)) {
                $cart->setCurrency($currency);
            }

            $cart->setOrderdate(new DateTime());
            $cart->setUserId($feUser->getUid());
        }

        $this->persistCart($cart);
        //$GLOBALS['TSFE']->fe_user->setKey('ses', 'shopCart', ''); // We don't need the cart in the session if we have a user

        return $cart;
    }

    public function findBySubmittedLimited($submitted, int $pid, $limited = null)
    {
        $query = $this->createQuery();
        $query->getQuerySettings()
        ->setRespectSysLanguage(false);
        //$query->getQuerySettings()->setLanguageOverlayMode(false);
        if ($limited != null && $limited > 0) {
            $query->setLimit($limited);
        }

        $query->getQuerySettings()
        ->setRespectStoragePage(false);
        $query->matching(
            $query->logicalAnd(
                $query->equals('submitted', $submitted),
                $query->equals('pid', $pid),
                $query->greaterThan('products', 0),
                $query->logicalNot($query->equals('paymentMethod', 0)),
                $query->logicalNot($query->equals('cartinfo', ''))
            )
        );

        return $query->execute();
    }

    public function getCartsFromUser($userId, $sorting = 'cartdate')
    {
        if ($userId > 0) {
            $query = $this->createQuery();
            $query->getQuerySettings()
            ->setRespectSysLanguage(false);
            //$query->getQuerySettings()->setLanguageOverlayMode(false);
            $query->matching($query->logicalAnd($query->equals('submitted', 1), $query->equals('userid', $userId)));
            $query->setOrderings(
                [
                    'cartdate' => QueryInterface::ORDER_DESCENDING,
                ]
            );
            return $queryResult = $query->execute();
        }

        return 0;
    }

    /**
     * @param Cart $cart
     */
    public function persistCart($cart): void
    {
        $context = GeneralUtility::makeInstance(Context::class);
        //enshures: cart is saved to db and session.shopCart=cart.uid
        $GLOBALS['TYPO3_CONF_VARS']['SYS']['cookieDomain'] = $_SERVER['HTTP_HOST'];

        if ($context->getPropertyFromAspect('frontend.user', 'isLoggedIn')) {
            $cart->setUserId($context->getPropertyFromAspect('frontend.user', 'id'));
        } else {
            $cart->setUserId(0);
        }

        if ($cart->_isNew()) {
            $this->add($cart);
        } else {
            $this->update($cart);
        }

        $this->persistenceManager->persistAll();
        $request = $GLOBALS['TYPO3_REQUEST'];
        $frontendUser = $request->getAttribute('frontend.user');
        if ($frontendUser) {
            $frontendUser->setKey('ses', 'shopCart', $cart->getUid());
            $frontendUser->storeSessionData();
        }

        //echo('session  cart is saved('.$cart->getUid().')<br />');
        $this->persistenceManager->persistAll();
    }

    /**
     * Suchfunktion für die Backend Bestelliste
     *
     * @param string $searchKeys Suchwort der Eingabemaske
     * @param array $uids
     * @param DateTime $vonDate
     * @param DateTime $bisDate
     * @param int $limited
     * @param boolean $carts
     * @return ObjectStorage<Cart>
     */
    public function findByCartInfoLimited(
        $limited,
        int $pid,
        $searchKeys = null,
        $uids = null,
        $vonDate = null,
        $bisDate = null,
        $carts = false
    ) {
        $query = $this->createQuery();

        $query->getQuerySettings()
        ->setRespectSysLanguage(false);
        //$query->getQuerySettings()->setLanguageOverlayMode(false);
        $query->getQuerySettings()
        ->setRespectStoragePage(false);

        if ($limited != null && $limited > 0) {
            $query->setLimit($limited);
        }

        $constraints = [];

        $constraints[] = $query->equals('submitted', !$carts);

        if ($searchKeys != null) {
            $constraints[] = $query->like('cartinfo', '%' . $searchKeys . '%');
        }

        if (count($uids) > 0) {
            $constraints[] = $query->in('uid', $uids);
        }

        if ($vonDate) {
            $constraints[] = $query->greaterThanOrEqual('tstamp', $vonDate->getTimestamp());
        }

        if ($bisDate) {
            $constraints[] = $query->lessThanOrEqual('tstamp', $bisDate->getTimestamp());
        }

        if ($pid > 0) {
            $constraints[] = $query->equals('pid', $pid);
        }

        $constraints[] = $query->greaterThan('products', 0);

        $constraints[] = $query->logicalNot($query->equals('paymentMethod', 0));

        $constraints[] = $query->logicalNot($query->equals('cartinfo', ''));

        $query->matching($query->logicalAnd(...$constraints));

        return $query->execute();
    }
}
