class IgShopProductComparisonHandler {
    constructor() {
        this.linkButtonTextSelector = '.link-button-text';
        this.ajaxLoaderSelector = '.ajax-loader';
        this.addProductLinkElement = document.querySelector('.igshop2-add-to-product-comparison-link');
        this.countWrapperElement = document.querySelector('.tx-igshop2-product-comparison-small');
        this.countTextElement = this.countWrapperElement ? this.countWrapperElement.querySelector('.igshop-product-comparison-count-text') : null;
        this.compareUrlElements = document.querySelectorAll('.shop-compare-lightbox-link');
        this.addProductLinkButtonTextElement = this.addProductLinkElement ? this.addProductLinkElement.querySelector(this.linkButtonTextSelector) : null;
        this.addProductAjaxLoaderElement = this.addProductLinkElement ? this.addProductLinkElement.querySelector(this.ajaxLoaderSelector) : null;
        this.productComparisonListElement = document.querySelector('.product-comparison-list');
        this.dropdownElement = document.querySelector('.product-comparison-dropdown');
        this.dropdownTextElement = this.dropdownElement ? this.dropdownElement.querySelector('.shop-compare-text') : null;
        this.comparisonLinkElement = document.querySelector('.tx-igshop2-product-comparison-small > a');
        this.lightbox = null;
        this.count = 0;
    }

    init() {
        if(this.countWrapperElement) {
            this.initProductCount();
        }
        if(this.addProductLinkElement) {
            this.addProductLinkElement.addEventListener('click', (e) => {
                e.preventDefault();

                this.addProduct();
            });
        }
        document.addEventListener('click', (e) => {
            let target = e.target.closest('.product-comparison-dropdown');
            if(!target) {
                this.hideDropdown();
            }
        });
        if(this.comparisonLinkElement) {
            this.comparisonLinkElement.addEventListener('click', (e) => {
                let compareProductsElement = document.querySelector('.compare-products');
                if(this.count < 2 || compareProductsElement) {
                    e.preventDefault();
                }
            });
        }
    }

    initProductCount() {
        let fetchProductsUrl = this.countWrapperElement.dataset.fetchProductsUrl;
        if(fetchProductsUrl) {
            let promise = null;
            let persistedProductComparisonUid = localStorage.getItem('igshopProductComparisonUid');
            if(persistedProductComparisonUid) {
                let requestData = new URLSearchParams({"tx_igshop2_productcomparison[productComparison]": persistedProductComparisonUid});
                promise = fetch(fetchProductsUrl, {method: 'POST', body: requestData});
            } else {
                promise = fetch(fetchProductsUrl, {method: 'POST'});
            }
            promise.then((response) => {
                return response.json();
            })
            .then((data) => {
                if(data.productComparisonUid) {
                    localStorage.setItem('igshopProductComparisonUid', data.productComparisonUid);
                }
                this.updateCount(data.productCount);
                this.updateCompareUrl(data.compareUrl);
            });
        }
    }

    addProduct() {
        let addProductUrl = this.addProductLinkElement.href;
        if(addProductUrl) {
            this.showAddProductAjaxLoader();
            
            let promise = null;
            let persistedProductComparisonUid = localStorage.getItem('igshopProductComparisonUid');
            
            if(persistedProductComparisonUid) {
                let requestData = new URLSearchParams({"tx_igshop2_productcomparison[productComparison]": persistedProductComparisonUid});
                promise = fetch(addProductUrl, {method: 'POST', body: requestData});
            } else {
                promise = fetch(addProductUrl, {method: 'POST'});
            }

            promise.then((response) => {
                return response.json();
            })
            .then((data) => {
                if(data.productComparisonUid) {
                    localStorage.setItem('igshopProductComparisonUid', data.productComparisonUid);
                }

                this.updateCount(data.productCount);
                this.updateCompareUrl(data.compareUrl);
                this.updateDropdown(data.html);
                this.showDropdown();

                this.hideAddProductAjaxLoader();
            });
        }
    }

    showAddProductAjaxLoader() {
        if(this.addProductAjaxLoaderElement) {
            this.addProductAjaxLoaderElement.style.display = '';
            this.addProductLinkButtonTextElement.style.visibility = 'hidden';
        }
    }

    hideAddProductAjaxLoader() {
        if(this.addProductAjaxLoaderElement) {
            this.addProductAjaxLoaderElement.style.display = 'none';
            this.addProductLinkButtonTextElement.style.visibility = '';
        }
    }

    // TODO: Analog zu smallFeCart Liste ausgeben und count aktualisieren
    updateView(html) {
        if(this.productComparisonListElement) {
            this.productComparisonListElement.innerHTML = html;
        }
    }

    updateCount(count) {
        this.count = count;
        if(this.countTextElement) {
            this.countTextElement.textContent = count;
        }
    }

    updateCompareUrl(compareUrl) {
        if(this.compareUrlElements) {
            this.compareUrlElements.forEach(function(compareUrlElement) {
                compareUrlElement.href = compareUrl;
            });
        }
    }

    updateDropdown(html) {
        if(this.dropdownTextElement) {
            this.dropdownTextElement.innerHTML = html;
        }
    }

    showDropdown() {
        if(this.dropdownElement) {
            this.dropdownElement.classList.add('show');
        }
    }

    hideDropdown() {
        if(this.dropdownElement) {
            this.dropdownElement.classList.remove('show');
        }
    }
}

let igShopProductComparisonHandler = new IgShopProductComparisonHandler();
igShopProductComparisonHandler.init();